const static = require('node-static');
const Metrics = require('anta');
const getHtml = require('anta/recipes/umi-perf/scripts/getHtml.js');
const path = require('path');
const fs = require('fs');
const detect = require('detect-port');
const http = require('http');
const vfs = require('vinyl-fs');
const gzip = require('gulp-gzip');
const map = require('map-stream');

const DEFAULT_PORT = 8080;
let entries = process.argv.slice(2);
let staticServer;
const dirname = process.cwd();
// const dirname = '/Users/yansheng/AntProjects/qiaohua-test-h5-app-new/.audit_reports/jezizb4d';
const distPath = path.join(dirname, 'dist');

function send(message) {
  if (process.send) {
    process.send(message);
  }
}

function saveLogs(logs) {
  const logFile = path.join(dirname, 'log.txt');
  let lg = '';
  if (fs.existsSync(logFile)) {
    lg = fs.readFileSync(logFile).toString() + '\n\n\n';
  }
  try {
    fs.writeFileSync(logFile, lg + logs.join('\n'));
  } catch (e) {
    console.log(e);
  }
}

function save(task) {
  try {
    const taskFile = path.join(dirname, 'task.json');
    fs.writeFileSync(taskFile, task);
    console.log(`save task: ${taskFile} done`);

    const reportFile = path.join(dirname, 'report.html');
    let html = getHtml(task);
    html = html.replace(/<body>/gi, '<body class="ide">');
    fs.writeFileSync(reportFile, html);
    console.log(`save report: ${reportFile} done`);
  } catch (e) {
    console.log('save error', e)
  }
}

function analysis(port) {
  const promises = entries.map((entry) => {
    return new Promise((resolve) => {
      const ts = (new Date()).getTime();
      console.log(`http://127.0.0.1:${port}${entry}`);
      new Metrics(`http://127.0.0.1:${port}${entry}`, {
        flags: {
          chromeFlags: ['--headless']
        },
      })
        .start()
        .then((resultJson) => {
          const jsFile = path.join(dirname, `result_${entry.replace('/', '_')}.js`);
          fs.writeFileSync(jsFile, `window.dealRawData('${entry}', ${JSON.stringify(resultJson[0], null, 2)});`);
          let score = 0;
          resultJson[0].runs[0].reportCategories.forEach(cate => {
            if (cate.name === 'Performance') {
              score = cate.score;
            }
          });

          resolve({
            success: true,
            name: entry,
            score,
            resultUrl: {
              url: `file://${jsFile}`,
            },
            takeTime: ((new Date()).getTime() - ts) / 1000,
          });
        })
        .catch((error) => {
          console.log(error);
          resolve({
            success: false,
            name: entry,
            error,
            takeTime: ((new Date()).getTime() - ts) / 1000,
          });
        });
    });
  });

  return Promise.all(promises)
    .then((values) => {
      save(JSON.stringify(values, null, 2));
      send({
        type: 'ANALYSIS_SUCCESS',
        payload: {
          success: true,
        },
      });
    }).catch((error) => {
      console.log(error);
      send({
        type: 'ANALYSIS_FAILED',
        payload: {
          success: false,
          error,
        },
      });
    });
}

function runner() {
  const _run = () => {
    detect(DEFAULT_PORT, (error, _port) => {
      if (error) {
        send({
          type: 'ANALYSIS_FAILED',
          payload: {
            success: false,
            error,
          },
        });
        return;
      }

      const fileServer = new static.Server(distPath, { gzip: true });
      staticServer = http.createServer((request, response) => {
        request.addListener('end', function() {
          fileServer.serve(request, response);
        }).resume();
      }).listen(_port, (err) => {
        if (err) {
          saveLogs(['listen error', err]);
          return;
        }
        console.log(`serve ${distPath} at http://127.0.0.1:${_port}`);
        analysis(_port).then(() => {
          saveLogs(['analysis done']);
          _exit();
        }).catch((error) => {
          saveLogs(['analysis catch', error]);
          _exit(-1);
        });
      });
    });
  };
  // gzip dist 下的 css/js 文件
  vfs.src(['**/*.js', '**/*.css'], { cwd: distPath })
    .pipe(gzip())
    .on('error', _run)
    .pipe(vfs.dest(distPath))
    .on('error', _run)
    .on('end', () => {
      _run()
    });
}

if (entries.length) {
  runner();
} else {
  // 查找目录下的 html 文件作为 entries
  vfs.src(['**/*.html'], { cwd: distPath, read: false, })
    .pipe(map((file, cb) => {
      entries.push('/' + file.relative.replace('\\', '/'));
      cb(null, file);
    }))
    .on('end', () => {
      runner();
    });
}

function _exit(code) {
  if (staticServer) {
    staticServer.close();
    staticServer = null;
  }
  process.exit(code || 0);
}

process.on('SIGINT', () => {
  saveLogs(['SIGINT']);
  _exit();
});






